# Klassifieds: A Klassifieds Advertising SystemK
# by Andrew Veitch
# Released under the GNU Public License
# $Id: Klassifieds.py,v 1.3 2002/11/29 16:57:37 andrew Exp $

__version__='0.23'

from Globals import DTMLFile
import Globals
from OFS.Folder import Folder
from Advert import Advert
from Products.ZCatalog.ZCatalog import ZCatalog
try:
    from Products.BTreeFolder.BTreeFolder import BTreeFolder
except ImportError:
    pass

manage_addKlassifiedsForm = DTMLFile("dtml/addKlassifiedsForm", globals())

def manage_addKlassifieds(self, id, title, AdminEmail, REQUEST, useBT = None):
    "Adds a Klassifieds advertising system to a folder"

    newKlassifieds=Klassifieds(id, title, AdminEmail, useBT)
    self._setObject(id, newKlassifieds)

    return self.manage_main(self, REQUEST)

class Klassifieds(Folder):
    "A Klassifieds advertising system"

    meta_type = "Klassifieds"
    icon = "misc_/OFSP/Folder_icon.gif"

    manage_options=(
        {'label':'Contents', 'action':'manage_main'},
        {'label':'Properties', 'action':'manage_propertiesForm',
            'help': ('Klassifieds', 'Properties.stx')},
        {'label':'Categories', 'action':'manage_categories',
            'help': ('Klassifieds', 'Categories.stx')},
        {'label':'Moderation', 'action':'manage_moderate',
            'help': ('Klassifieds', 'Moderation.stx')},
        {'label':'Banned', 'action':'manage_banned',
            'help': ('Klassifieds', 'Banned.stx')},
        {'label':'Maillist', 'action':'manage_maillist'},
        {'label':'Expire', 'action':'manage_expire',
            'help': ('Klassifieds', 'Expire.stx')},
        {'label':'View', 'action':'index_html'}
    )

    def __init__(self, id, title, AdminEmail, useBT):
        self.id  = id
        self.title = title
        self.AdminEmail = AdminEmail
        self.LastAdvertID = 0
        self.Moderate = ""
        self.Pagecounter = "on"
        self.Spamguard = ""
        self.Verifyemail = ""
        self.MaxPicBytes = "32768"
        self.MaxPicWidth = 200
        self.MaxPicHeight = 200
        self.CategoryList= ['Automobiles', 'Announcements', 'CDs, Tapes & DVDs',
                'Computers', 'Jobs', 'Personals', 'Real Estate', 'Other']
        self.CategoryDescription = ['Cars, trucks, SUVs, vans',
                'Births, Weddings, Funerals, Local Events',
                'All types of music and films',
                'Computer hardware, software and peripherals',
                'Full time & part time jobs',
                'Meet local people for friendship or romance',
                'Residential and commercial property for sale or rent',
                'Anything else for sale or wanted']
        self.FieldLabels = ['State', 'Zip', 'Country']
        self.BannedList = []
        if useBT:
            self._setObject('adverts', BTreeFolder())
            self.adverts.id = 'advert'
            self._setObject('photos', BTreeFolder())
            self.photos.id = 'photos'
        else:
            self.manage_addFolder(id='adverts', title='')
            self.manage_addFolder(id='photos', title='')
        self._addDTML('standard_html_header', '', 'standard_html_header')
        self._addDTML('standard_html_footer', '', 'standard_html_footer')
        self._addDTML('advert_footer', '', 'advert_footer')
        self._addDTML('showCategory', '', 'showCategory')
        self._addDTML('index_html', '', 'index_html')
        self._addDTML('help', '', 'help')
        self._addDTML('addAdvertForm', '', 'addAdvertForm')
        self._initCatalog()

    def _addDTML(obj,id,title,file):
        path=Globals.package_home(globals())
        f=open(path + '/dtml/'+file+'.dtml')
        file=f.read()
        f.close()
        obj.manage_addDTMLMethod(id,title,file)
        return getattr(obj,id)

    def _initCatalog(self):
        "Init the Catalog & setup the indices & metadata"
        if not hasattr(self, 'Catalog'):
            self._setObject('Catalog',
                     ZCatalog('Catalog', title='', container=self))
        self.catalogPath = "Catalog"
        self.Catalog.addIndex('Categories', 'KeywordIndex')
        self.Catalog.addIndex('Posted', 'FieldIndex')
        self.Catalog.addIndex('Maillist', 'FieldIndex')
        self.Catalog.addIndex('Approved', 'FieldIndex')
        self.Catalog.addIndex('Verified', 'FieldIndex')
        self.Catalog.addIndex('Photo', 'FieldIndex')
        self.Catalog.addIndex('AdvertText', 'TextIndex')
        self.Catalog.addIndex('Expires', 'FieldIndex')
        self.Catalog.addColumn('id')
        self.Catalog.addColumn('title')
        self.Catalog.addColumn('Photo')
        self.Catalog.addColumn('Posted')
        self.Catalog.addColumn('Verified')
        self.Catalog.addColumn('Email')

    def manage_addBannedList(self, Email, REQUEST):
        "Add an email address to the banned list"
        if Email:
            self.BannedList.append(Email)
            self.__changed__(1)
        return self.manage_banned(self, REQUEST)

    def manage_deleteBannedList(self, Emails, REQUEST):
        "Delete email addresses from the banned list"
        for email in Emails:
            i = self.BannedList.index(email)
            del self.BannedList[i]
        self.__changed__(1)
        return self.manage_banned(self, REQUEST)

    def manage_addCategory(self, Category, Description, REQUEST):
        "Adds a new Category & Description"
        if Category:
            self.CategoryList.append(Category)
            self.CategoryDescription.append(Description)
            self.__changed__(1)
        return self.manage_categories(self, REQUEST)

    def manage_deleteCategory(self, Categories, REQUEST):
        "Deletes categories. Called by editCategories."
        for cat in Categories:
            i = self.CategoryList.index(cat)
            del self.CategoryList[i]
            del self.CategoryDescription[i]
        self.__changed__(1) 
        
        return self.manage_categories(self, REQUEST)

    def manage_approveAdvert(self, ads, REQUEST):
        "Called from the moderateAdvert management screen"
        for ad in ads:
            self.adverts[ad].setApproved()
        return self.manage_main(self, REQUEST)

    def manage_deleteAdvert(self, ads, REQUEST):
        "Used by the Moderation and Expirey management screens"
        for ad in ads:
            if self.adverts[ad].Photo:
                self.photos.manage_delObjects(ad)
        self.adverts.manage_delObjects(ads)
        return self.manage_main(self, REQUEST)

    def manage_deleteAdvertBan(self, ads, REQUEST):
        "Delete adverts and add advertiser to the banned list"
        for ad in ads:
            self.BannedList.append(self.adverts[ad].Email)
        self.__changed__(1)
        return self.manage_deleteAdvert(ads, REQUEST)

    def manage_properties(self, title, AdminEmail, MaxPicBytes, MaxPicWidth, 
            MaxPicHeight, REQUEST, Moderate = "", Pagecounter = "", 
            Spamguard = "", Verifyemail = ""):
        "A method to edit Klassifieds properties"
        self.title = title
        self.AdminEmail = AdminEmail
        self.Moderate = Moderate
        self.Pagecounter = Pagecounter
        self.Spamguard = Spamguard
        self.Verifyemail = Verifyemail
        self.MaxPicBytes = MaxPicBytes
        self.MaxPicWidth = MaxPicWidth
        self.MaxPicHeight = MaxPicHeight
        return self.manage_main(self, REQUEST)

    def addAdvert(self, Name, Street, City, State, Zip, Telephone, Country,
            Email, Website, Categories, title, AdvertText, Password, Expires,
            Posted, Display = "", Maillist = "", REQUEST=None):
        "Method to add an advert to the Klassifieds system"
        id = self.LastAdvertID = self.LastAdvertID + 1
        id = str(id)
        if self.Moderate:
            Approved = ""
        else:
            Approved = "on"
        if self.Verifyemail:
            Verified = ""
        else:
            Verified = "on"
        advert = Advert(id, Name, Street, City, State, Zip, Telephone, Country,
            Email, Website, Display, Categories, title, AdvertText, Password, 
            Expires, Maillist, Posted, Approved, Verified)
        self.adverts._setObject(id, advert)

        if REQUEST is not None:
            REQUEST.set('id', id)
            return self.adverts[id].addPhotoForm(self, REQUEST)

    # Management pages
    manage_propertiesForm = DTMLFile("dtml/manage_propertiesForm", globals())
    manage_categories = DTMLFile("dtml/manage_categories", globals())
    manage_moderate = DTMLFile("dtml/manage_moderate", globals())
    manage_expire = DTMLFile("dtml/manage_expire", globals())
    manage_maillist = DTMLFile("dtml/manage_maillist", globals())
    manage_banned = DTMLFile("dtml/manage_banned", globals())

    # Display pages
    previewAdvert = DTMLFile("dtml/previewAdvert", globals())
    searchForm = DTMLFile("dtml/searchForm", globals())
    getMaillist = DTMLFile("dtml/getMaillist", globals())
    left_gif = DTMLFile("dtml/left.gif", globals())
    right_gif = DTMLFile("dtml/right.gif", globals())
